@@ -3,7 +3,7 @@ use clap::Parser;
 use crate::git::remote::{ConnectionMethod, RepoHostName};
 
 #[derive(Parser, Debug)]
-#[clap(name = "jointhedots", bin_name = "jtd", about)]
+#[clap(name = "jointhedots", bin_name = "jtd", about, version)]
 pub enum JoinTheDots {
     Install(InstallSubcommandArgs),
     Sync(SyncSubcommandArgs),
@@ -11,7 +11,7 @@ pub enum JoinTheDots {
 }
 
 #[derive(clap::Args, Debug)]
-#[clap(about = "Install a specified JTD repository", author, version)]
+#[clap(about = "Install a specified JTD repository", version)]
 pub struct InstallSubcommandArgs {
     #[clap(help = "The location of the repository in the form USERNAME/REPONAME")]
     pub repository: String,
@@ -25,6 +25,14 @@ pub struct InstallSubcommandArgs {
     )]
     pub method: ConnectionMethod,
 
+    #[clap(
+        long = "manifest",
+        short = 'n',
+        help = "The manifest to use in the repository",
+        default_value = "jtd.yaml"
+    )]
+    pub manifest: String,
+
     #[clap(
         help = "The dotfiles to install. If unspecified, install all of them",
         conflicts_with = "all"
@@ -66,16 +74,25 @@ pub struct InstallSubcommandArgs {
 #[derive(clap::Args, Debug)]
 #[clap(
     about = "Sync the currently installed JTD repository with the provided remote repo.",
-    author,
     version
 )]
 pub struct SyncSubcommandArgs {
     #[clap(help = "The location of the repository in the form USERNAME/REPONAME")]
     pub repository: String,
 
-    #[clap(help = "The dotfiles to sync. If unspecified, sync all of them")]
+    #[clap(
+        help = "The dotfiles to sync. If unspecified, sync all of them",
+        conflicts_with = "all"
+    )]
     pub target_dotfiles: Vec<String>,
 
+    #[clap(
+        help = "Whether to install all dotfiles in the config",
+        long = "all",
+        short = 'a'
+    )]
+    pub all: bool,
+
     #[clap(
         arg_enum,
         long = "method",
@@ -85,6 +102,14 @@ pub struct SyncSubcommandArgs {
     )]
     pub method: ConnectionMethod,
 
+    #[clap(
+        long = "manifest",
+        short = 'n',
+        help = "The manifest to use in the repository",
+        default_value = "jtd.yaml"
+    )]
+    pub manifest: String,
+
     #[clap(
         arg_enum,
         default_value = "GitHub",
@@ -102,5 +127,5 @@ pub struct SyncSubcommandArgs {
 }
 
 #[derive(clap::Args, Debug)]
-#[clap(about = "Interactively install dotfiles", author, version)]
+#[clap(about = "Interactively install dotfiles", version)]
 pub struct InteractiveSubcommandArgs {}
